/*
 * Copyright (c) 2009, Morten Nobel-Jrgensen All Rights Reserved.
 *
 * GNU General Public License, version 2
 * http://www.gnu.org/licenses/gpl-2.0.txt
 *
 */

package com.mortennobel.easizer.gui;

import static com.mortennobel.easizer.i18n.EasizerTextConstants.getString;
import com.mortennobel.easizer.model.EasizerWorker;
import com.mortennobel.easizer.model.GUISetup;
import com.mortennobel.easizer.model.ResizeConfiguration;
import com.mortennobel.imagescalingframework.ProgressListener;
import net.iharder.dnd.FileDrop;
import org.jdesktop.application.Application;
import org.jdesktop.application.LocalStorage;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.IOException;
import java.io.ObjectOutputStream;
import java.io.OutputStream;
import java.net.URI;
import java.net.URISyntaxException;
import java.util.ArrayList;

public class EasizerController implements ActionListener, FileDrop.Listener {
    private final EasizerGUI easizerGUI;

    private final GUISetup guiSetup;
    private ResizeConfiguration configuration;

	public void saveSettings() {
		LocalStorage localStorage = Application.getContext().getLocalStorage();
        try{
            OutputStream outputStream = localStorage.openOutputFile(".easizer.gui");
            ObjectOutputStream oos = new ObjectOutputStream(outputStream);
            oos.writeObject(getGuiSetup());
            oos.writeObject(getConfiguration());
            oos.close();
            outputStream.close();
        }
        catch (Exception e){
            e.printStackTrace();
        }
	}

	static class ResizeTask extends SwingWorker<Void, Void> {
        private final File[] files;
        private final EasizerGUI easizerGUI;

		private int currentFileIndex;

		private ResizeTask(File[] files, EasizerGUI easizerGUI) {
            this.files = files;
            this.easizerGUI = easizerGUI;
        }

        private String currentFile;
        private java.util.List<Exception> allExceptions = new ArrayList<Exception>();

		public int getNumberOfFiles(){
			return files.length;
		}

		public int getCurrentFileIndex(){
			return currentFileIndex;
		}

		public String getLongestFilename(){
			String name = files[0].getName();
			for (int i=1;i<files.length;i++){
				String name2 = files[i].getName();
				if (name2.length()>name.length()){
					name = name2;
				}
			}
			return name;
		}

		@Override
        public Void doInBackground() {
            ResizeConfiguration resizeConfig = easizerGUI.getResizeConfiguration();
            try {
                final int progressParts = 100/files.length;
                String prefix = resizeConfig.overwriteFile()?"":getString("FileResizedPrefix")+"_"+System.currentTimeMillis()+"_";

                for (currentFileIndex=0;currentFileIndex<files.length;currentFileIndex++){
                    File inputFile = files[currentFileIndex];
                    currentFile = inputFile.getName();
                    if (currentFileIndex==0){
                        setProgress(0);
                    }
                    final int totalProgress = progressParts*currentFileIndex;
                    setProgress(totalProgress);
                    File outputFile = new File(inputFile.getParent(), prefix+inputFile.getName());
                    java.util.List<Exception> exceptions = EasizerWorker.resizeImage(inputFile, outputFile, resizeConfig,new ProgressListener() {
                        public void notifyProgress(float fraction) {
							int progress = (int)(totalProgress+progressParts*fraction);
							assert progress>=0;
							assert progress<=100;
							setProgress(progress);
                        }
                    });
                    allExceptions.addAll(exceptions);
                }
                setProgress(100);
            } catch (Exception e) {
                e.printStackTrace();
            }
            return null;
        }

        @Override
        public void done() {
            for (Exception e:allExceptions){
                JOptionPane.showMessageDialog(null, "<html><b>"+getString("ExceptionDialogHeadline")+"</b><br>"+e.getMessage(),getString("ExceptionDialogTitle") , JOptionPane.ERROR_MESSAGE);
            }
        }

        public String getCurrentFile() {
            return currentFile;
        }
    }

    public EasizerController(EasizerGUI easizerGUI, GUISetup guiSetup, ResizeConfiguration configuration) {

        this.easizerGUI = easizerGUI;
        this.guiSetup = guiSetup;
        this.configuration = configuration;
    }

    public void actionPerformed(ActionEvent e) {
        if (e.getSource() == easizerGUI.alwaysOnTop){
            JFrame frame = easizerGUI.getMainTopLevel();
            if (!frame.isAlwaysOnTopSupported()){
                JOptionPane.showMessageDialog(frame, getString("AlwaysOnTopNotSupported"));
                easizerGUI.alwaysOnTop.setSelected(false);
                easizerGUI.alwaysOnTop.setEnabled(false);
            }
            else{
                frame.setAlwaysOnTop(easizerGUI.alwaysOnTop.isSelected());
                guiSetup.setAlwaysOnTop(easizerGUI.alwaysOnTop.isSelected());
            }
        }
		else if (e.getSource()==easizerGUI.settings){
			showSettings();
		}
		else if (e.getSource()== easizerGUI.exit){
            easizerGUI.exit();
        }
        else if (e.getSource() == easizerGUI.helpTopics){
            if (Desktop.isDesktopSupported()) {
                try {
                    Desktop.getDesktop().browse(new URI("http://www.nobel-joergensen.com/java/projects/easizer/helptopics.jsp"));
                } catch (IOException e1) {
                    e1.printStackTrace();
                } catch (URISyntaxException e1) {
                    e1.printStackTrace();
                }
            }
        }
        else if (e.getSource()==easizerGUI.about){
			EasizerAbout.show(easizerGUI.getMainTopLevel());
        }
    }

	private void showSettings() {
		boolean okPressed = EasizerSettingsGUI.showResizeConfiguration(easizerGUI.getMainTopLevel(),configuration);
		if (okPressed){
			saveSettings();
		}
	}

	public GUISetup getGuiSetup() {
        return guiSetup;
    }

    public ResizeConfiguration getConfiguration() {
        return configuration;
    }

	public void setConfiguration(ResizeConfiguration configuration) {
		this.configuration = configuration;
	}

    public void filesDropped(File[] files) {
		if (easizerGUI.getResizeConfiguration().overwriteFile() && easizerGUI.getResizeConfiguration().isPromptBeforeOverwriteFiles()){
			int res = JOptionPane.showConfirmDialog(easizerGUI.getMainTopLevel(), getString("ConfirmOverwriteFileBody"),getString("ConfirmOverwriteFileHeader"),JOptionPane.YES_NO_OPTION,JOptionPane.WARNING_MESSAGE);
			if (res==JOptionPane.NO_OPTION){
				JOptionPane.showMessageDialog(easizerGUI.getMainTopLevel(), getString("ChangeSettingToNewCopy"));
				return;
			}
		}
		ResizeTask resizeTask = new ResizeTask(files, easizerGUI);
        new EasizerProgressMonitor(easizerGUI.getMainTopLevel(), resizeTask);
        resizeTask.execute();
    }
}
