/*
 * Copyright (c) 2009, Morten Nobel-Jrgensen All Rights Reserved.
 *
 * GNU General Public License, version 2
 * http://www.gnu.org/licenses/gpl-2.0.txt
 *
 */

package com.mortennobel.easizer.gui;

import javax.swing.*;
import javax.swing.border.EmptyBorder;
import javax.imageio.ImageIO;
import java.awt.*;
import java.awt.image.BufferedImage;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.ItemListener;
import java.awt.event.ItemEvent;
import java.io.IOException;

import net.miginfocom.swing.MigLayout;
import com.mortennobel.easizer.model.ResizeAlgorithm;
import com.mortennobel.easizer.model.ResizeConfiguration;
import static com.mortennobel.easizer.i18n.EasizerTextConstants.*;

/**
 * Settings
 */
public class EasizerSettingsGUI extends JDialog implements ActionListener {

	private String BEST_QUALITY = getString("SettingsScalingSimpleQuality");
	private String HIGH_SPEED = getString("SettingsScalingSimpleSpeed");

	private ItemListener resizeItemListener = new ItemListener() {
		public void itemStateChanged(ItemEvent e) {
			if (e==null || e.getSource()==unsharpenMask ||  e.getStateChange()==ItemEvent.SELECTED){
				String title = getResizeAlgorithm().toString()+(unsharpenMask.isSelected()?" - "+getString("SettingsPreviewUnsharpenMask"):"");
				scalingAlgorithmExample.setImageFilter(getResizeAlgorithm().getResizeAlgorithm(testImageSize.width, testImageSize.height,unsharpenMask.isSelected()), title);
			}
		}
	};

	private DefaultComboBoxModel simpleOptions = new DefaultComboBoxModel(new Object[]{BEST_QUALITY,HIGH_SPEED});
	private ResizeAlgorithm simpleAlgorithmBestQuality = ResizeAlgorithm.BiCubic;
	private ResizeAlgorithm simpleAlgorithmHighSpeed = ResizeAlgorithm.BilinearMultistep;
	private DefaultComboBoxModel advancedOptions = new DefaultComboBoxModel(new Object[]{
			ResizeAlgorithm.Bell,
			ResizeAlgorithm.BiCubic,
			ResizeAlgorithm.BiCubicHighFreq,
			ResizeAlgorithm.Box,
			ResizeAlgorithm.BSpline,
			ResizeAlgorithm.Hermite,
			ResizeAlgorithm.Lanczos3,
			ResizeAlgorithm.Mitchell,
			ResizeAlgorithm.Triangle,
			ResizeAlgorithm.BiCubicMultistep,
			ResizeAlgorithm.BilinearMultistep});

	private JCheckBox showAdvancedScalingOptions = new JCheckBox(getString("SettingsShowAdvScaleOptions"));
	private JCheckBox unsharpenMask = new JCheckBox(getString("SettingsUseUnsharpenMask"));
	private JComboBox scalingOptions = new JComboBox();
	private ExampleImagePanel scalingAlgorithmExample;

	private final Dimension testImageSize = new Dimension(160,102);

	private boolean okPressed = false;

	private class LayoutClass extends JPanel{
		private LayoutClass() {
			super(new MigLayout("","",""));
		}

		private void addContent(String headling, JComponent content){
			addContent(headling,null, content);
		}

		private void addContent(String headline, String description, final JComponent content){
			JPanel leftPanel = new JPanel();
			leftPanel.setLayout(new BoxLayout(leftPanel, BoxLayout.Y_AXIS));
			JLabel headlineLabel = new JLabel(headline);
			headlineLabel.setAlignmentX(JComponent.LEFT_ALIGNMENT);
			leftPanel.add(headlineLabel);

			if (description!=null){
				JLabel descriptionLabel = new JLabel("<html>"+description+"</html>");
				Dimension preferredSize = GUIUtil.getPreferredSize(descriptionLabel.getText(), true,150);
				descriptionLabel.setPreferredSize(preferredSize);
				descriptionLabel.setBorder(new EmptyBorder(0,10,0,0));

				Font font = descriptionLabel.getFont();
				descriptionLabel.setFont(font.deriveFont(font.getSize2D()-4));

				descriptionLabel.setAlignmentX(JComponent.LEFT_ALIGNMENT);

				leftPanel.add(descriptionLabel);
			}
			leftPanel.add(Box.createVerticalBox());

			add(leftPanel);

			if (content!=null){
				JPanel rightPanel = new JPanel();
				rightPanel.setLayout(new BoxLayout(rightPanel, BoxLayout.Y_AXIS));
				content.setAlignmentX(JComponent.LEFT_ALIGNMENT);
				rightPanel.add(content);
				rightPanel.add(Box.createVerticalBox());
				add(rightPanel, "wrap");
			}
			else{
				add(new JPanel(), "wrap");
			}
		}
	}

	private final JCheckBox promptBeforeOverwritingFiles = new JCheckBox(getString("SettingsAlwaysPromptOnOverwrite"));
	private JComponent generalPanel = createGeneralPanel();
	private JComponent scalingPanel = createScalingPanel();

	private JToggleButton generalButton = new JToggleButton(getString("SettingsGeneral"), new ImageIcon(getClass().getResource("/images/buttonicon/cog.png")));
	private JToggleButton scalingButton = new JToggleButton(getString("SettingsScaling"), new ImageIcon(getClass().getResource("/images/buttonicon/picture.png")));

	private JButton okButton = new JButton(getString("Ok"));
	private JButton cancelButton = new JButton(getString("Cancel"));

	private JScrollPane scrollPane;

	private EasizerSettingsGUI(JFrame owner){
		super(owner,getString("SettingsTitle"),true);
		initGUI();
	}

	private void initGUI() {

		resizeItemListener.itemStateChanged(null); // invoke update of image

		getContentPane().setLayout(new BorderLayout());
		JPanel categoryBar = new JPanel(new FlowLayout(FlowLayout.LEFT));

		generalButton.addActionListener(this);
		scalingButton.addActionListener(this);
		//appearanceButton.addActionListener(this);

		generalButton.setVerticalTextPosition(AbstractButton.BOTTOM);
		generalButton.setHorizontalTextPosition(AbstractButton.CENTER);
		scalingButton.setVerticalTextPosition(AbstractButton.BOTTOM);
		scalingButton.setHorizontalTextPosition(AbstractButton.CENTER);
		// appearanceButton.setVerticalTextPosition(AbstractButton.BOTTOM);
		// appearanceButton.setHorizontalTextPosition(AbstractButton.CENTER);

		scrollPane = new JScrollPane(generalPanel, JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);

		Dimension scrollPaneDim = getMaxPreferredDimensionDimension(categoryBar.getPreferredSize(), generalPanel);
		scrollPaneDim = getMaxPreferredDimensionDimension(scrollPaneDim, scalingPanel);
		scrollPaneDim.width += 10;
		scrollPaneDim.height += 10;
		scrollPane.setMinimumSize(scrollPaneDim);
		scrollPane.setPreferredSize(scrollPaneDim);

		getContentPane().add(scrollPane, BorderLayout.CENTER);

		categoryBar.add(generalButton);
		categoryBar.add(scalingButton);
		// categoryBar.add(appearanceButton);
		ButtonGroup buttonGroup = new ButtonGroup();
		buttonGroup.add(generalButton);
		buttonGroup.add(scalingButton);
		// buttonGroup.add(appearanceButton);
		getContentPane().add(categoryBar, BorderLayout.NORTH);

		JPanel buttonPanel = new JPanel(new FlowLayout(FlowLayout.RIGHT));

		buttonPanel.add(okButton);
		buttonPanel.add(cancelButton);
		okButton.addActionListener(this);
		cancelButton.addActionListener(this);
		getContentPane().add(buttonPanel , BorderLayout.SOUTH);
	}

	private JComponent createGeneralPanel(){
		LayoutClass result = new LayoutClass();
		result.addContent(getString("SettingsOverwriteCategory"), getString("SettingsOverwriteDescription"), promptBeforeOverwritingFiles);
		return result;
	}

	private JComponent createScalingPanel(){
		BufferedImage exampleImage;
		try {
			exampleImage = ImageIO.read(getClass().getResource("/images/exampleimage.jpg"));
		} catch (IOException e) {
			e.printStackTrace();
			exampleImage = new BufferedImage(testImageSize.width,testImageSize.height, BufferedImage.TYPE_INT_RGB);
		}
		scalingAlgorithmExample = new ExampleImagePanel(exampleImage, testImageSize);

		LayoutClass result = new LayoutClass();
		scalingOptions.setPreferredSize(getScalingOptionsPreferredSize());
		scalingOptions.setModel(advancedOptions);
		scalingOptions.addItemListener(resizeItemListener);

		unsharpenMask.addItemListener(resizeItemListener);

		showAdvancedScalingOptions.addItemListener(new ItemListener() {
			public void itemStateChanged(ItemEvent e) {
				DefaultComboBoxModel model = showAdvancedScalingOptions.isSelected()?advancedOptions : simpleOptions;
				scalingOptions.setModel(model);
				scalingOptions.repaint();
				resizeItemListener.itemStateChanged(null);
			}
		});

		result.addContent(getString("SettingsScalingMethodCategory"), "", scalingOptions);
		result.addContent("", "", showAdvancedScalingOptions);
		result.addContent(getString("SettingsPostEffectCategory"), getString("SettingsPostEffectDescription"), unsharpenMask);
		result.addContent(getString("SettingsScalingExample"),"", scalingAlgorithmExample);
		return result;
	}

	private Dimension getScalingOptionsPreferredSize(){
		ComboBoxModel currentModel = scalingOptions.getModel();
		scalingOptions.setModel(advancedOptions);
		Dimension advPrefSize = new Dimension(scalingOptions.getPreferredSize());
		scalingOptions.setModel(simpleOptions);
		Dimension simplePrefSize = new Dimension(scalingOptions.getPreferredSize());
		scalingOptions.setModel(currentModel);
		return new Dimension(Math.max(advPrefSize.width,advPrefSize.width),
				Math.max(advPrefSize.height,simplePrefSize.height));
	}

	public ResizeAlgorithm getResizeAlgorithm(){
		Object selectedObject = scalingOptions.getSelectedItem();

		if (selectedObject==BEST_QUALITY){
			return simpleAlgorithmBestQuality;
		}
		else if (selectedObject==HIGH_SPEED){
			return simpleAlgorithmHighSpeed;
		}
		assert selectedObject instanceof ResizeAlgorithm:"Was "+selectedObject;
		return (ResizeAlgorithm) selectedObject;
	}

	private Dimension getMaxPreferredDimensionDimension(Dimension dim, JComponent component){
		Dimension preferredDim = component.getPreferredSize();
		return new Dimension(Math.max(dim.width, preferredDim.width), Math.max(dim.height,preferredDim.height));
	}

	public void actionPerformed(ActionEvent e) {
		Object source = e.getSource();
		if (source instanceof JToggleButton){
			JComponent newContent;
			if (source == generalButton){
				newContent = generalPanel;
			}
			else {
				newContent = scalingPanel;
			}

			scrollPane.setViewportView(newContent);
			scrollPane.invalidate();
		}
		else if (source == okButton){
			okPressed = true;
			dispose();
		}
		else if (source == cancelButton){
			dispose();
		}
	}

	public static boolean showResizeConfiguration(JFrame owner, ResizeConfiguration config){
		EasizerSettingsGUI gui = new EasizerSettingsGUI(owner);
		gui.setConfig(config);
		gui.pack();
		Point point = new Point(owner.getLocationOnScreen());
		point.translate(20,20);
		gui.setLocation(point);
		gui.setVisible(true);
		if (gui.okPressed){
			config.setAdvancedOptions(gui.showAdvancedScalingOptions.isSelected());
			config.setUnsharpenFilter(gui.unsharpenMask.isSelected());
			config.setResizeAlgorithmId(gui.getResizeAlgorithm().ordinal());
			config.setPromptBeforeOverwriteFiles(gui.promptBeforeOverwritingFiles.isSelected());
		}
		return gui.okPressed;
	}

	private void setConfig(ResizeConfiguration config) {
		showAdvancedScalingOptions.setSelected(config.isAdvancedOptions());
		if (config.isAdvancedOptions()){
			scalingOptions.setModel(advancedOptions);
			scalingOptions.setSelectedItem(config.getResizeAlgorithm());
		} else {
			scalingOptions.setModel(simpleOptions);
			if (config.getResizeAlgorithm()==simpleAlgorithmHighSpeed){
				scalingOptions.setSelectedItem(HIGH_SPEED);
			}
			else{
				scalingOptions.setSelectedItem(BEST_QUALITY);
			}
		}
		unsharpenMask.setSelected(config.isUnsharpenFilter());
		resizeItemListener.itemStateChanged(null);

		promptBeforeOverwritingFiles.setSelected(config.isPromptBeforeOverwriteFiles());
		
	}
}
