/*
 * Copyright (c) 2009, Morten Nobel-Jrgensen All Rights Reserved.
 *
 * GNU General Public License, version 2
 * http://www.gnu.org/licenses/gpl-2.0.txt
 *
 */

package com.mortennobel.easizer.gui;

import static com.mortennobel.easizer.i18n.EasizerTextConstants.getString;

import javax.imageio.ImageIO;
import javax.swing.*;
import javax.swing.border.LineBorder;
import javax.swing.border.TitledBorder;
import java.awt.*;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.image.BufferedImage;
import java.awt.image.BufferedImageOp;
import java.io.IOException;

public class ExampleImagePanel extends JComponent {

	private BufferedImage zoomInIcon;
	private BufferedImage zoomOutIcon;

	private final BufferedImage sourceImage;
	private BufferedImage dstImage;

	private TitledBorder titledBorder = new TitledBorder(new LineBorder(Color.black), " ", TitledBorder.DEFAULT_JUSTIFICATION, TitledBorder.ABOVE_BOTTOM);


	private int imageLocationX;
	private int imageLocationY;

	private int zoom = 1;
	private int zoomMargin = 3;

	public ExampleImagePanel(BufferedImage sourceImage, Dimension prefferedImage) {
		try {
			zoomInIcon = ImageIO.read(getClass().getResource("/images/buttonicon/zoom_in.png"));
			zoomOutIcon = ImageIO.read(getClass().getResource("/images/buttonicon/zoom_out.png"));
		} catch (IOException e) {
			e.printStackTrace();
		}
		this.sourceImage = sourceImage;
		Font font = titledBorder.getTitleFont();
		font = font.deriveFont(font.getSize()-4f);
		titledBorder.setTitleFont(font);
		setBorder(titledBorder);
		Insets ins = getInsets();
		Dimension dim = new Dimension(prefferedImage.width+ins.left+ins.right,prefferedImage.height+ins.top+ins.bottom);
		setMinimumSize(dim);
		setPreferredSize(dim);

		updateTooltip();

		MouseAdapter mouseAdapter = new MouseAdapter() {
			public void mouseClicked(MouseEvent e) {
				if (dstImage!=null){
					Insets ins = getInsets();
					if (e.getX()>ins.left+dstImage.getWidth()-zoomMargin*2-zoomInIcon.getWidth()){
						if (e.getY()<ins.top+zoomInIcon.getWidth()+zoomMargin){
							zoom++;
							imageLocationX += dstImage.getWidth()/zoom;
							imageLocationY += dstImage.getHeight()/zoom;
							keepImageWithinBounds();
						}
						else if (e.getY()<ins.top+zoomInIcon.getWidth()*2+zoomMargin*2){
							zoom = Math.max(1,zoom-1);
							imageLocationX -= dstImage.getWidth()/zoom;
							imageLocationY -= dstImage.getHeight()/zoom;
							keepImageWithinBounds();
						}
					}
					updateTooltip();
					repaint();
				}
			}

			private int mousePressedStartX;
			private int mousePressedStartY;
			private int mouseStartX;
			private int mouseStartY;

			public void mousePressed(MouseEvent e) {
				mousePressedStartX = e.getX();
				mousePressedStartY = e.getY();
				mouseStartX = imageLocationX;
				mouseStartY = imageLocationY;
			}

			public void mouseDragged(MouseEvent e) {
				int deltaX = mousePressedStartX-e.getX();
				int deltay = mousePressedStartY-e.getY();

				imageLocationX =  mouseStartX+deltaX;
				imageLocationY = mouseStartY+deltay;
				keepImageWithinBounds();
				if (zoom>1){
					repaint();
				}
			}

			public void mouseMoved(MouseEvent e) {

			}

			private void keepImageWithinBounds(){
				int maxX = dstImage.getWidth()*zoom-dstImage.getWidth();
				int maxY = dstImage.getHeight()*zoom-dstImage.getHeight();
				imageLocationX = Math.max(0,Math.min(maxX,imageLocationX));
				imageLocationY = Math.max(0,Math.min(maxY,imageLocationY));
			}
		};

		addMouseListener(mouseAdapter);
		addMouseMotionListener(mouseAdapter);
	}

	private void updateTooltip(){
		if (zoom>1){
			setToolTipText(getString("SettingsPreviewDragTooltip"));
			setCursor(new Cursor(Cursor.MOVE_CURSOR));
		}
		else{
			setToolTipText(null);
			setCursor(new Cursor(Cursor.HAND_CURSOR));
		}
	}

	public void paintComponent(Graphics g) {
		if (dstImage!=null){
			Insets ins = getInsets();

			int x = ins.left + (getWidth()-ins.left-ins.right- dstImage.getWidth())/2;
			int y = ins.top + (getHeight()-ins.top-ins.bottom- dstImage.getHeight())/2;
			if (zoom>1){
				Graphics2D g2 = (Graphics2D) g.create();
				g2.translate(x,y);
				g2.setClip(0,0,dstImage.getWidth(),dstImage.getHeight());
				g2.setRenderingHint(RenderingHints.KEY_INTERPOLATION, RenderingHints.VALUE_INTERPOLATION_NEAREST_NEIGHBOR);

				g2.drawImage(dstImage, - imageLocationX,-imageLocationY,dstImage.getWidth()*zoom,dstImage.getHeight()*zoom,null);
				g2.dispose();
			}
			else{
				g.drawImage(dstImage, x,y,null);
			}
			g.setColor(new Color(1,1,1,0.75f));

			g.fillRect(x+dstImage.getWidth()-zoomInIcon.getWidth()-zoomMargin*2,y,zoomInIcon.getWidth()+zoomMargin*2,zoomInIcon.getHeight()*2+3*zoomMargin);
			g.drawImage(zoomInIcon, -zoomMargin+x+dstImage.getWidth()-zoomInIcon.getWidth(),zoomMargin+y,null);
			g.drawImage(zoomOutIcon, -zoomMargin+x+dstImage.getWidth()-zoomInIcon.getWidth(),zoomMargin*2+y+zoomOutIcon.getHeight(),null);
		}
	}

	public void setSize(int width, int height) {
		super.setSize(width, height);
	}

	public void setImageFilter(final BufferedImageOp bufferedImageOp, final String title){
		Thread thread = new Thread(){
			public void run(){
				dstImage = bufferedImageOp.filter(sourceImage, null);
				setTitle(title);

			}
		};
		thread.start();
	}

	private void setTitle(final String title){
		SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				titledBorder.setTitle(title);
				repaint();
			}
		});
	}
}
