/*
 * Copyright (c) 2009, Morten Nobel-Jrgensen All Rights Reserved.
 *
 * GNU General Public License, version 2
 * http://www.gnu.org/licenses/gpl-2.0.txt
 *
 */

package com.mortennobel.easizer.model;

import com.mortennobel.imagescalingframework.AdvancedResizeOp;
import com.mortennobel.imagescalingframework.ProgressListener;

import javax.imageio.*;
import javax.imageio.metadata.IIOMetadata;
import javax.imageio.plugins.jpeg.JPEGImageWriteParam;
import javax.imageio.stream.ImageInputStream;
import javax.imageio.stream.ImageOutputStream;
import javax.imageio.stream.MemoryCacheImageOutputStream;
import java.awt.image.BufferedImage;
import java.io.*;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

/**
 *
 */
public class EasizerWorker {
    public static List<Exception> resizeImage(File input, File output, ResizeConfiguration config, final ProgressListener progressListener){
        List<Exception> exceptions = new ArrayList<Exception>();
        try {
			progressListener.notifyProgress(0);
            int maxWidth = config.getMaxWidth();
            int maxHeight = config.getMaxHeight();
            if (maxHeight>maxWidth){
                // swap
                int tmp = maxWidth;
                //noinspection SuspiciousNameCombination
                maxWidth = maxHeight;
                maxHeight = tmp;
            }

            boolean copyMetaData = true;
            Iterator readerIter = ImageIO.getImageReadersByFormatName("jpeg");
            ImageReader jpegImageReader = (ImageReader) readerIter.next();
            boolean seekForwardOnly = true;
            boolean ignoreMetadata = false;

            ImageInputStream iis = ImageIO.createImageInputStream(new BufferedInputStream(new FileInputStream(input)));
            jpegImageReader.setInput(iis, seekForwardOnly, ignoreMetadata);
            BufferedImage bi = jpegImageReader.read(0);

            IIOMetadata imageMetadata = null;

            if (copyMetaData){
                imageMetadata = getMetadata(jpegImageReader, input, iis);
            }

            if (bi.getHeight()>bi.getWidth()){
                // swap maxWidth and maxHeiht
                // swap
                int tmp = maxWidth;
                //noinspection SuspiciousNameCombination
                maxWidth = maxHeight;
                maxHeight = tmp;
            }
            BufferedImage res;
            if (maxWidth!=Integer.MAX_VALUE && maxHeight!=Integer.MAX_VALUE && config.isMaxDimension()){
                double scaleFactorWidth = maxWidth/(double)bi.getWidth();
                double scaleFactorHeight = maxHeight/(double)bi.getHeight();

                double scaleFactor = Math.min(scaleFactorWidth,scaleFactorHeight);

				int newWidth = (int)(bi.getWidth()*scaleFactor);
				int newHeigh = (int)(bi.getHeight()*scaleFactor);

				AdvancedResizeOp resampleOp = config.getResizeAlgorithm().getResizeAlgorithm(newWidth,newHeigh,config.isUnsharpenFilter());
				ProgressListener resampleProgressListener;
				if (config.isMaxFileSize()){
					resampleProgressListener = new ProgressListener() {
						public void notifyProgress(float fraction) {
							progressListener.notifyProgress(fraction/2);
						}
					};
				}
				else{
					resampleProgressListener = progressListener;
				}

				resampleOp.addProgressListener(resampleProgressListener);
				res = resampleOp.filter(bi,null);
			}
            else{
                res = bi;
            }

            byte[] resByteArray;
            if (config.isMaxFileSize()){
                int maxSizeBytes = config.getFileSizeBytes();
				ProgressListener resizeProgressListener;
				if (config.isMaxDimension()){
					resizeProgressListener = new ProgressListener() {
						public void notifyProgress(float fraction) {
							progressListener.notifyProgress(0.5f+fraction/2f);
						}
					};
				}
				else{
					resizeProgressListener = progressListener;
				}
				resByteArray = compressImage(resizeProgressListener, imageMetadata, res, maxSizeBytes);
            }
            else{
                Iterator iter = ImageIO.getImageWritersByFormatName("jpeg");
                ImageWriter writer = (ImageWriter)iter.next();
                JPEGImageWriteParam iwp = (JPEGImageWriteParam) writer.getDefaultWriteParam();

				float currentCompression = config.getDefaultJPEGQuality();

                resByteArray = createImage(imageMetadata,res, writer, iwp, currentCompression);
            }
            FileOutputStream fos = null;
            try{
                fos = new FileOutputStream(output);
                fos.write(resByteArray);
            }
            finally{
                fos.close();
            }
        } catch (Exception e) {
            e.printStackTrace();
            exceptions.add(e);
        }
        return exceptions;
    }

    /**
     * Save the image with a certain max size
     *
     * @param progressListener
     * @param imageMetadata
     * @param res
     * @param maxSizeBytes
     * @return
     * @throws IOException
     */
    private static byte[] compressImage(ProgressListener progressListener, IIOMetadata imageMetadata, BufferedImage res, int maxSizeBytes) throws IOException {
        byte[] resByteArray;
        int iterations = 15;
        float progressFractions = 1f/(iterations+4);

        progressListener.notifyProgress(progressFractions);
        Iterator iter = ImageIO.getImageWritersByFormatName("jpeg");
        ImageWriter writer = (ImageWriter)iter.next();
        JPEGImageWriteParam iwp = (JPEGImageWriteParam) writer.getDefaultWriteParam();
        float max = 1f;
        float min=0.001f;
        float mid = min+(max-min)/2;
        byte[] maxByteArray = createImage(imageMetadata,res, writer, iwp, max);
        progressListener.notifyProgress(progressFractions*2);
        byte[] minByteArray = createImage(imageMetadata, res, writer, iwp, min);
        progressListener.notifyProgress(progressFractions*3);
        byte[] midByteArray = createImage(imageMetadata, res, writer, iwp, mid);
        progressListener.notifyProgress(progressFractions*4);
        for (int i=0;i<iterations;i++){
            if (maxByteArray.length<maxSizeBytes){
                i=iterations;
            }
            else{
                if (midByteArray.length>maxSizeBytes){
                    max=mid;
                    maxByteArray=midByteArray;
                }
                else{
                    min=mid;
                    minByteArray=midByteArray;
                }
                mid = min+(max-min)/2;
                midByteArray = createImage(imageMetadata, res, writer, iwp, mid);
                progressListener.notifyProgress(progressFractions*(5+i));
            }
        }
        resByteArray = maxByteArray.length<maxSizeBytes?maxByteArray:midByteArray.length<maxSizeBytes?midByteArray : minByteArray;
        return resByteArray;
    }

    /**
     * Bug fix from http://bugs.sun.com/bugdatabase/view_bug.do?bug_id=4924909
     * @param jpegImageReader
     * @param input
     * @param iis
     * @return
     * @throws IOException
     */
    private static IIOMetadata getMetadata(ImageReader jpegImageReader, File input, ImageInputStream iis) throws IOException {
        IIOMetadata imageMetadata = null;
        try{
            imageMetadata = jpegImageReader.getImageMetadata(0);
        } catch (javax.imageio.IIOException iioex){
            if(iioex.getMessage() != null &&
                           iioex.getMessage().endsWith("without prior JFIF!")
                    ){
                iis.close();
                iis = patch(input);
                jpegImageReader.setInput(iis);
                imageMetadata = jpegImageReader.getImageMetadata(0);
            }
        }
        iis.close();
        
        return imageMetadata;
    }

    /** Patches a JPEG file that is missing a JFIF marker **/
        private static class PatchInputStream extends FilterInputStream{
            private static final int[] JFIF = {0xFF,
                0xE0, 0x00, 0x10, 0x4A, 0x46, 0x49,
                0x46, 0x00, 0x01, 0x02, 0x00, 0x00,
                0x01, 0x00, 0x01, 0x00, 0x00};
            int position = 0;

            public PatchInputStream(InputStream in){
                super(in);
            }

            public int read() throws IOException{
                int result;
                if(position < 2){
                    result = in.read();
                } else if (position < 2 + JFIF.length){
                    result = JFIF[position - 2];
                } else {
                    result = in.read();
                }
                position++;
                return result;
            }

            public int read(byte[] b, int off, int len)
            throws IOException{
                final int max = off + len;
                int bytesread = 0;
                for(int i=off; i<max; i++){
                    final int bi = read();
                    if(bi == -1){
                        if(bytesread == 0){
                            bytesread = -1;
                        }
                        break;
                    } else {
                        b[i] = (byte) bi;
                        bytesread ++;
                    }
                }
                return bytesread;
            }

        }

        private static ImageInputStream patch(File jpeg) throws IOException{
            InputStream in = new FileInputStream(jpeg);
            in = new BufferedInputStream(in);
            in = new PatchInputStream(in);
            return ImageIO.createImageInputStream(in);
        }


    private static byte[] createImage(IIOMetadata imageMetadata, BufferedImage res, ImageWriter writer, JPEGImageWriteParam iwp, float quality) throws IOException {
        if (quality!=-1){
            iwp.setCompressionMode(ImageWriteParam.MODE_EXPLICIT);
            iwp.setCompressionQuality(quality);   // an float between 0 and 1
        }
        iwp.setOptimizeHuffmanTables(true); // fixes Missing Huffman code table entry
        ByteArrayOutputStream baos = new ByteArrayOutputStream();
        ImageOutputStream output = new MemoryCacheImageOutputStream(baos);
        writer.setOutput(output);
        IIOImage image = new IIOImage(res, null, imageMetadata);
        writer.write(imageMetadata, image, iwp);

        return baos.toByteArray();
    }

    
}

