/*
 * Copyright (c) 2009, Morten Nobel-Jrgensen All Rights Reserved.
 *
 * GNU General Public License, version 2
 * http://www.gnu.org/licenses/gpl-2.0.txt
 *
 */

package com.mortennobel.easizer.model;

import java.io.Serializable;

public class ResizeConfiguration implements Serializable {
    private static final long serialVersionUID = 142102576L;

    public static enum FileSizeUnit {
        Bytes(1, "B"),
        Kilobyte(1000, "kB"),
        Megabyte(1000*1000, "MB"),
        Kibibyte(1024, "KiB"),
        Mebibyte(1024*1024, "MiB")
        ;
        private final int unit;
        private final String unitShort;

        FileSizeUnit(int unit, String unitShort) {
            this.unit = unit;
            this.unitShort = unitShort;
        }

        public int getUnit() {
            return unit;
        }

        public String getUnitShort() {
            return unitShort;
        }
    }

    private boolean maxDimension;
    private int maxWidth;
    private int maxHeight;
    private boolean maxFileSize;
    private int fileSize;
    private FileSizeUnit fileSizeUnit;
    private boolean overwriteFile;
	private int resizeAlgorithmId;
	private boolean unsharpenFilter;
	private float defaultJPEGQuality;
	private boolean advancedOptions;
	private boolean promptBeforeOverwriteFiles = true;
	

	public ResizeConfiguration() {
        // default values
        this.maxDimension = false;
        this.maxWidth = 300;
        this.maxHeight = 200;
        this.maxFileSize = true;
        this.fileSize = 300;
        this.fileSizeUnit = FileSizeUnit.Kilobyte;
        this.overwriteFile = true;
		this.resizeAlgorithmId = ResizeAlgorithm.BiCubic.ordinal();
		this.unsharpenFilter = false;
		this.defaultJPEGQuality = 0.8f;
		this.advancedOptions = false;
		promptBeforeOverwriteFiles = true;
	}
    
    public ResizeConfiguration(boolean maxDimension,
							   int maxWidth, int maxHeight, boolean maxFileSize, int fileSize, FileSizeUnit fileSizeUnit,
							   boolean overwriteFile,int resizeAlgorithmId,boolean unsharpenFilter,float defaultJPEGQuality,
							   boolean advancedOptions, boolean promptBeforeOverwriteFiles) {
        this.maxDimension = maxDimension;
        this.maxWidth = maxWidth;
        this.maxHeight = maxHeight;
        this.maxFileSize = maxFileSize;
        this.fileSize = fileSize;
        this.fileSizeUnit = fileSizeUnit;
        this.overwriteFile = overwriteFile;
		this.resizeAlgorithmId = resizeAlgorithmId;
		this.unsharpenFilter = unsharpenFilter;
		this.defaultJPEGQuality = defaultJPEGQuality;
		this.advancedOptions = advancedOptions;
		this.promptBeforeOverwriteFiles = promptBeforeOverwriteFiles;
	}

	public boolean isOverwriteFile() {
		return overwriteFile;
	}

	public void setOverwriteFile(boolean overwriteFile) {
		this.overwriteFile = overwriteFile;
	}

	public int getResizeAlgorithmId() {
		return resizeAlgorithmId;
	}

	public ResizeAlgorithm getResizeAlgorithm(){
		return ResizeAlgorithm.values()[resizeAlgorithmId];
	}

	public void setResizeAlgorithmId(int resizeAlgorithmId) {
		this.resizeAlgorithmId = resizeAlgorithmId;
	}

	public boolean isUnsharpenFilter() {
		return unsharpenFilter;
	}

	public void setUnsharpenFilter(boolean unsharpenFilter) {
		this.unsharpenFilter = unsharpenFilter;
	}

	public float getDefaultJPEGQuality() {
		return defaultJPEGQuality;
	}

	public void setDefaultJPEGQuality(float defaultJPEGQuality) {
		this.defaultJPEGQuality = defaultJPEGQuality;
	}

	public String getConfigString(){
        StringBuilder sb = new StringBuilder();
        if (maxDimension){
            sb.append(" -width ");
            sb.append(maxWidth);
            sb.append(" -height ");
            sb.append(maxHeight);
        }
        if (maxFileSize){
            sb.append(" -fileSize ");
            sb.append(fileSize);
            sb.append(" -fileSizeUnit ");
            sb.append(fileSizeUnit);
        }
        return sb.toString();
    }

    public boolean isMaxDimension() {
        return maxDimension;
    }

	

	public void setMaxDimension(boolean maxDimension) {
        this.maxDimension = maxDimension;
    }

    public int getMaxWidth() {
        return maxWidth;
    }

    public void setMaxWidth(int maxWidth) {
        this.maxWidth = maxWidth;
    }

    public int getMaxHeight() {
        return maxHeight;
    }

    public void setMaxHeight(int maxHeight) {
        this.maxHeight = maxHeight;
    }

    public boolean isMaxFileSize() {
        return maxFileSize;
    }

    public void setMaxFileSize(boolean maxFileSize) {
        this.maxFileSize = maxFileSize;
    }

    public int getFileSize() {
        return fileSize;
    }

    public int getFileSizeBytes(){
        return fileSize * fileSizeUnit.getUnit();
    }

    public void setFileSize(int fileSize) {
        this.fileSize = fileSize;
    }

    public FileSizeUnit getFileSizeUnit() {
        return fileSizeUnit;
    }

    public void setFileSizeUnit(FileSizeUnit fileSizeUnit) {
        this.fileSizeUnit = fileSizeUnit;
    }

    public boolean overwriteFile(){
        return overwriteFile;
    }

	public boolean isAdvancedOptions() {
		return advancedOptions;
	}

	public void setAdvancedOptions(boolean advancedOptions) {
		this.advancedOptions = advancedOptions;
	}

	public boolean isPromptBeforeOverwriteFiles() {
		return promptBeforeOverwriteFiles;
	}

	public void setPromptBeforeOverwriteFiles(boolean promptBeforeOverwriteFiles) {
		this.promptBeforeOverwriteFiles = promptBeforeOverwriteFiles;
	}
}
